@@ -1,7 +1,7 @@
 from unittest.mock import MagicMock, Mock
 
-from jaseci.svc import MailService
-from jaseci.svc.mail import MAIL_CONFIG
+from jaseci import JsOrc
+from jaseci.svc.mail_svc import MailService
 from jaseci.utils.test_core import CoreTest
 
 
@@ -11,22 +11,23 @@ class MailLibTest(CoreTest):
     fixture_src = __file__
 
     def __init__(self, *args, **kwargs):
-        MAIL_CONFIG["enabled"] = True
+        JsOrc.settings("MAIL_CONFIG")["enabled"] = True
         MailService.connect = MagicMock(return_value=Mock())
         super(MailLibTest, self).__init__(*args, **kwargs)
 
     def setUp(self):
-        super().setUp(True)
+        super().setUp()
 
-    def test_send_mail(self):
+    @JsOrc.inject(services=["mail"])
+    def test_send_mail(self, mail: MailService):
         self.call(
             self.mast,
             ["sentinel_register", {"code": self.load_jac("mail_test.jac")}],
         )
         ret = self.call(self.mast, ["walker_run", {"name": "send_mail"}])
         self.assertTrue(ret["success"])
-        self.assertTrue(self.mast._h.mail.connect.called)
+        self.assertTrue(mail.connect.called)
         self.assertEqual(
-            self.mast._h.mail.app.method_calls[0].args,
+            mail.app.method_calls[0].args,
             (None, ["jaseci.dev@gmail.com"], "Test Subject", ("Test", "<h1>Test</h1>")),
         )
