@@ -4,6 +4,8 @@ Interpreter for jac code in AST form
 This interpreter should be inhereted from the class that manages state
 referenced through self.
 """
+from jaseci import JsOrc
+from jaseci.svc.task_svc import TaskService
 from jaseci.utils.utils import is_jsonable, parse_str_token, uuid_re
 from jaseci.element.element import Element
 from jaseci.graph.node import Node
@@ -20,6 +22,7 @@ from jaseci.jac.machine.jac_value import jac_elem_unwrap as jeu
 from jaseci.jac.machine.jac_value import jac_wrap_value as jwv
 from copy import copy, deepcopy
 from base64 import b64decode
+from itertools import pairwise
 
 from jaseci.jac.jsci_vm.op_codes import JsCmp
 
@@ -794,12 +797,14 @@ class Interp(VirtualMachine):
             elif kid[0].name == "KW_SYNC":
                 self.run_atom(kid[1])
                 val = self.pop()
-                task_func = self._h.task
-                if not task_func.is_running():
-                    raise Exception("Task hook is not yet initialized!")
                 self.push(
                     JacValue(
-                        self, value=task_func.get_by_task_id(val.value["result"], True)
+                        self,
+                        value=(
+                            JsOrc.svc("task")
+                            .poke(TaskService)
+                            .get_by_task_id(val.value["result"], True)
+                        ),
                     )
                 )
             else:
@@ -1154,6 +1159,13 @@ class Interp(VirtualMachine):
                     result = JacValue(
                         self, value=atom_res.value.index(min(atom_res.value))
                     )
+                elif op == "pairwise":
+                    result = JacValue(
+                        self, value=[list(s) for s in pairwise(atom_res.value)]
+                    )
+                elif op == "unique":
+                    result = JacValue(self, value=list(set(atom_res.value)))
+
                 elif len(kid) < 2 and op == "pop":
                     result = JacValue(self, value=atom_res.value.pop())
                     self.candidate_writethrough()
