@@ -6,6 +6,14 @@ class InterpreterTest(CoreTest):
 
     fixture_src = __file__
 
+    def test_quick_check(self):
+        ret = self.call(
+            self.mast,
+            ["sentinel_register", {"code": self.load_jac("lang_features.jac")}],
+        )
+        ret = self.call(self.mast, ["walker_run", {"name": "quick_check"}])
+        self.assertEqual(ret["report"][0], "edge5")
+
     def test_has_var_plucking(self):
         ret = self.call(
             self.mast,
